import {getAssets} from "./assets.ts";
import {Point, Size} from "../datatypes.ts";


export class Sprite {
  readonly imageSet: string;
  // spritesheet params
  readonly pixelsPerSubimage: Size;
  readonly origin: Point;
  readonly cellsPerSheet: Size;
  // number of frames
  readonly nFrames: number;

  constructor(imageSet: string, pixelsPerSubimage: Size, origin: Point, cellsPerSheet: Size, nFrames: number) {
    this.imageSet = imageSet;
    this.pixelsPerSubimage = pixelsPerSubimage;
    this.origin = origin;
    this.cellsPerSheet = cellsPerSheet;
    this.nFrames = nFrames;

    let nPossibleFrames = this.cellsPerSheet.w * this.cellsPerSheet.h;
    if (this.nFrames > nPossibleFrames) {
      throw `can't have ${this.nFrames} with a spritesheet dimension of ${nPossibleFrames}`;
    }
  }

  internalDraw(ctx: CanvasRenderingContext2D, {position, ix, xScale, yScale, angle}: {position: Point, ix?: number, xScale?: number, yScale?: number, angle?: number}) {
    ix = ix == undefined ? 0 : ix;
    xScale = xScale == undefined ? 1.0 : xScale;
    yScale = yScale == undefined ? 1.0 : yScale;
    angle = angle == undefined ? 0.0 : angle;

    // ctx.translate(Math.floor(x), Math.floor(y));
    ctx.translate(position.x, position.y);
    ctx.rotate(angle * Math.PI / 180);
    ctx.scale(xScale, yScale);
    ctx.translate(-this.origin.x, -this.origin.y);

    let me = getAssets().getImage(this.imageSet);
    let srcCx = ix % this.cellsPerSheet.w;
    let srcCy = Math.floor(ix / this.cellsPerSheet.w);
    let srcPx = srcCx * this.pixelsPerSubimage.w;
    let srcPy = srcCy * this.pixelsPerSubimage.h;
    ctx.drawImage(me, srcPx, srcPy, this.pixelsPerSubimage.w, this.pixelsPerSubimage.h, 0, 0, this.pixelsPerSubimage.w, this.pixelsPerSubimage.h);
  }
}
