import {BG_OUTER} from "./colors.ts";
import {D, I} from "./engine/public.ts";
import {IGame, Point, Size} from "./engine/datatypes.ts";
import {HuntMode} from "./huntmode.ts";
import {getPageLocation, getPartLocation, Page, UIPart} from "./layout.ts";
import {getHud} from "./hud.ts";

class MenuCamera {
  // measured in whole screens
  position: Point;
  target: Point;

  constructor({position, target}: {position: Point, target: Point}) {
    this.position = position;
    this.target = target;
  }

  update() {
    let adjust = (x0: number, x1: number) => {
      if (Math.abs(x1 - x0) < 0.01) { return x1; }
      return (x0 * 8 + x1 * 2) / 10;
    }
    this.position = new Point(
      adjust(this.position.x, this.target.x),
      adjust(this.position.y, this.target.y),
    );
  }
}
export class Game implements IGame {
  camera: MenuCamera;
  page: Page;
  huntMode: HuntMode;

  constructor() {
    this.camera = new MenuCamera({
      position: new Point(0, 0),
      target: new Point(0, 0),
    });
    this.page = "Gameplay";

    this.huntMode = HuntMode.generate({depth: 1});
  }

  update() {
    if (I.isKeyPressed("w")) {
      this.page = "Gameplay"
    }
    if (I.isKeyPressed("s")) {
      this.page = "Thralls"
    }

    this.camera.target = getPageLocation(this.page);
    D.camera = new Point(
      D.size.w * this.camera.position.x,
      D.size.h * this.camera.position.y,
    )
    this.camera.update();

    // state-specific updates
    this.updateGameplay();
  }

  draw() {
    // draw screen background
    let oldCamera = D.camera;
    D.camera = new Point(0, 0);
    D.fillRect(new Point(0, 0), D.size, BG_OUTER);
    D.camera = oldCamera;

    this.drawGameplay();

    // we draw all states at once and pan between them
    // mainFont.drawText({ctx: ctx, text: "You have been given a gift.", x: 0, y: 0})
    let mouse = I.mousePosition?.offset(D.camera);
    if (mouse != null) {
      D.invertRect(mouse.offset(new Point(-1, -1)), new Size(3, 3))
    }
  }

  #withCamera(part: UIPart, cb: () => void) {
    let region = getPartLocation(part);

    D.withCamera(D.camera.offset(region.top.negate()), cb)
  }

  updateGameplay() {
    this.#withCamera("Gameplay", () => {
      this.huntMode.update();
    });
    this.#withCamera("HUD", () => { getHud().update() })
  }

  drawGameplay() {
    this.#withCamera("Gameplay", () => {
      this.huntMode.draw();
    });
    this.#withCamera("HUD", () => { getHud().draw() })
  }
}


export let game = new Game();