import {AlignX, AlignY, Point, Rect, Size} from "./engine/datatypes.ts";
import {D} from "./engine/public.ts";
import {getHud} from "./hud.ts";

// general
let margin = 8;
export function getLayoutRect(
  size: Size,
  options?: {alignX?: AlignX, alignY?: AlignY}
): Rect {
  let {w: screenW, h: screenH} = D.size;

  // first of all: place the _internal_ screen inside the real screen
  let marginalScreenW = screenW - margin * 2;
  let marginalScreenH = screenH - margin * 2;
  let marginalScreenX = margin;
  let marginalScreenY = margin;

  // NOTE: If the screen is too small, remainingSpace will be negative
  // This is fine -- it actually results in reasonable outcomes except
  // that the size of the box is exceeded in the opposite of the align direction.
  let {w: innerW, h: innerH} = size;
  let remainingSpaceX = marginalScreenW - innerW;
  let remainingSpaceY = marginalScreenH - innerH;
  if (options?.alignY == AlignY.Bottom) {
    console.log(`Remaining space y: ${remainingSpaceY}`)
  }

  let alignXCoef =
    options?.alignX == AlignX.Left ? 0.0 :
    options?.alignX == AlignX.Center ? 0.5 :
    options?.alignX == AlignX.Right ? 1.0 :
    0.5;
  let alignYCoef =
    options?.alignY == AlignY.Top ? 0.0 :
    options?.alignY == AlignY.Middle ? 0.5 :
    options?.alignY == AlignY.Bottom ? 1.0 :
    0.5;

  let x = marginalScreenX + alignXCoef * remainingSpaceX;
  let y = marginalScreenY + alignYCoef * remainingSpaceY;

  return new Rect(
    new Point(Math.floor(x), Math.floor(y)),
    size
  )
}

// specific
export type Page = "Gameplay" | "Thralls";
export type UIPart = "HUD" | "Gameplay" | "Thralls";

export function getPartPage(part: UIPart): Page {
  switch (part) {
    case "HUD":
    case "Gameplay":
      return "Gameplay";
    case "Thralls":
      return "Thralls";
  }

  throw `invalid part: ${part}`
}

export function getPageLocation(page: Page): Point {
  // NOTE: Measured in screens, not pixels
  switch (page) {
    case "Gameplay":
      return new Point(0, 0);
    case "Thralls":
      return new Point(0, 1);
  }

  throw `invalid page: ${page}`
}

export function getPartLocation(part: UIPart): Rect {
  // TODO: in pixels, not screens
  let {w: screenW, h: screenH} = D.size;
  let pageOffset = getPageLocation(getPartPage(part));
  let layoutRect = internalGetPartLayoutRect(part);

  return layoutRect.offset(new Point(
    pageOffset.x * screenW,
    pageOffset.y * screenH
  ));

}

export function internalGetPartLayoutRect(part: UIPart) {
  switch (part) {
    case "Gameplay":
    case "Thralls":
      return getLayoutRect(new Size(384, 384));
    case "HUD":
      return getLayoutRect(getHud().size, {
        alignX: AlignX.Left,
        alignY: AlignY.Middle
      })
  }
  throw `not sure what layout rect to use ${part}`
}