import {Stat} from "./datatypes.ts";

export class PlayerProgress {
  #stats: Record<Stat, number>
  #blood: number

  constructor() {
    this.#stats = {
      AGI: 10,
      INT: 10,
      CHA: 10,
      PSI: 10,
    }
    this.#blood = 0;

    this.refill();
  }

  refill() {
    this.#blood = 2000;
  }

  add(stat: Stat, amount: number) {
    if (amount != Math.floor(amount)) {
      throw `stat increment must be integer: ${amount}`
    }
    if (amount <= 0) {
      throw `stat increment must be >0: ${amount}`
    }
    this.#stats[stat] += amount;
  }

  getStat(stat: Stat): number {
    return this.#stats[stat]
  }

  getBlood(): number {
    return Math.floor(Math.max(this.#blood, 0));
  }

  spendBlood(amt: number) {
    this.#blood -= amt;
  }
}

let active: PlayerProgress = new PlayerProgress();

export function getPlayerProgress(): PlayerProgress {
  return active
}